/*
 *   This file is part of PKSM-Core
 *   Copyright (C) 2016-2020 Bernardo Giordano, Admiral Fish, piepie62
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Additional Terms 7.b and 7.c of GPLv3 apply to this file:
 *       * Requiring preservation of specified reasonable legal notices or
 *         author attributions in that material or in the Appropriate Legal
 *         Notices displayed by works containing it.
 *       * Prohibiting misrepresentation of the origin of that material,
 *         or requiring that modified versions of such material be marked in
 *         reasonable ways as different from the original version.
 */

#include "sav/SavDP.hpp"
#include "wcx/PGT.hpp"

namespace pksm
{
    SavDP::SavDP(std::shared_ptr<u8[]> dt) : Sav4(dt, 0x80000)
    {
        game = Game::DP;

        GBOOffset = 0xC0EC;
        SBOOffset = 0x1E2CC;
        GBO();
        SBO();

        Trainer1        = 0x64 + gbo;
        Party           = 0x98 + gbo;
        PokeDex         = 0x12DC + gbo;
        WondercardFlags = 0xA6D0 + gbo;
        WondercardData  = 0xA7FC + gbo;
        PouchHeldItem   = 0x624 + gbo;
        PouchKeyItem    = 0x8B8 + gbo;
        PouchTMHM       = 0x980 + gbo;
        MailItems       = 0xB10 + gbo;
        PouchMedicine   = 0xB40 + gbo;
        PouchBerry      = 0xBE0 + gbo;
        PouchBalls      = 0xCE0 + gbo;
        BattleItems     = 0xD1C + gbo;
        Box             = 0xC104 + sbo;
    }

    std::map<Sav::Pouch, std::vector<int>> SavDP::validItems() const
    {
        return {
            {Pouch::NormalItem,
                {68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88,
                    89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106,
                    107, 108, 109, 110, 111, 135, 136, 213, 214, 215, 216, 217, 218, 219, 220, 221,
                    222, 223, 224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237,
                    238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253,
                    254, 255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 267, 268, 269,
                    270, 271, 272, 273, 274, 275, 276, 277, 278, 279, 280, 281, 282, 283, 284, 285,
                    286, 287, 288, 289, 290, 291, 292, 293, 294, 295, 296, 297, 298, 299, 300, 301,
                    302, 303, 304, 305, 306, 307, 308, 309, 310, 311, 312, 313, 314, 315, 316, 317,
                    318, 319, 320, 321, 322, 323, 324, 325, 326, 327}},
            {Pouch::KeyItem, {428, 429, 430, 431, 432, 433, 434, 435, 436, 437, 438, 439, 440, 441,
                                 442, 443, 444, 445, 446, 447, 448, 449, 450, 451, 452, 453, 454,
                                 455, 456, 457, 458, 459, 460, 461, 462, 463, 464}},
            {Pouch::TM,
                {328, 329, 330, 331, 332, 333, 334, 335, 336, 337, 338, 339, 340, 341, 342, 343,
                    344, 345, 346, 347, 348, 349, 350, 351, 352, 353, 354, 355, 356, 357, 358, 359,
                    360, 361, 362, 363, 364, 365, 366, 367, 368, 369, 370, 371, 372, 373, 374, 375,
                    376, 377, 378, 379, 380, 381, 382, 383, 384, 385, 386, 387, 388, 389, 390, 391,
                    392, 393, 394, 395, 396, 397, 398, 399, 400, 401, 402, 403, 404, 405, 406, 407,
                    408, 409, 410, 411, 412, 413, 414, 415, 416, 417, 418, 419, 420, 421, 422, 423,
                    424, 425, 426, 427}},
            {Pouch::Mail, {137, 138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148}},
            {Pouch::Medicine,
                {17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37,
                    38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54}},
            {Pouch::Berry, {149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159, 160, 161, 162,
                               163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175, 176,
                               177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190,
                               191, 192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204,
                               205, 206, 207, 208, 209, 210, 211, 212}},
            {Pouch::Ball, {1, 2, 3, 4, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16}},
            {Pouch::Battle, {55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67}}};
    }
}
